#include "queue.h"

/* Malloc wrapper */
void *Malloc(size_t size)
{	void *ptr;
	if((ptr = malloc(size)) != NULL)
	{	return ptr;
	}
	else
	{	fprintf(stderr, "malloc: memory request failed\n");
		exit(1);
	}	
}

/* queue_alloc */
void *
queue_alloc()
{	queue_t *q = (queue_t *) Malloc(sizeof(queue_t));
	queue_init(q);
	return q;
}

/*queue_init()*/
void
queue_init(queue_t *q)
{	q->size = 0;
	q->head = q->tail = NULL;
}

/* queue_enqueue */
void
queue_enqueue(queue_t *q, void *elem)
{	//printf("enqueue: %p\n", elem); fflush(stdout);
	queue_node *tmp = (queue_node *)Malloc(sizeof(queue_node));
	tmp->next = NULL;
	tmp->data = elem;
	if(q->tail)
	{	q->tail->next = tmp;
		q->tail = tmp;
	} else {
		q->head = q->tail = tmp;
	}
	q->size++;
}

/* queue_dequeue */
void *
queue_dequeue(queue_t *q)
{	void *entry;
	queue_node *tmp;
	if(q->head)
	{	tmp = q->head;
		q->head = q->head->next;
		if(q->head == NULL) q->tail = NULL;
		entry = tmp->data;
		//printf("dequeue: %p\n", entry); fflush(stdout);
		free(tmp);
	} else {	
		fprintf(stderr, "queue: dequeue: queue empty\n");
		exit(1);
	}
	q->size--;
	return entry;
}

/* queue_empty */
int
queue_empty(const queue_t *q)
{	return (q->head == NULL);
}

/* queue_free */
void
queue_free(queue_t *q)
{	if(q)
	{	while(q->head)
		{	queue_dequeue(q);
		}
		free(q);
	}
}

void
queue_print(const queue_t *q)
{	queue_node *tmp = q->head;
	while(tmp)
	{	printf("entry: %p\n", tmp->data); fflush(stdout);
		tmp = tmp->next;
	}
}
